from builtins import input
from datetime import datetime
import json
import logging
import os
from os import path
import shutil
import subprocess
import sys

from splunk.clilib.bundle_paths import make_splunkhome_path

BACKUP_STATE_ACHIEVED = False
CURRENT_TIME = datetime.today().strftime('%Y-%m-%d-%H:%M')
FAILURE_FLAG = False
FAILURE_MESSAGES = []
UPGRADE_STATUS_FILE = "vmware_metric_ta_upgrade_status.json"
# The structure of the JSON file
# {
#     "time": <Time when the last upgrade occured>,
#     "errors": <List of errors occured while performing upgrade>,
#     "upgrade_successfull": <Boolean flag indicating the status of last upgrade>,
#     "backup_state_reached": <Boolean flag indicating whether Backup status was reached or not in last upgrade>
# }

SCH_VMWARE_BLACKLIST = ["inputs.conf", "hydra_metadata.conf",
                        "hydra_session.conf", "ta_vmware_cache.conf"]
SCH_HYDRA_BLACKLIST = ["inputs.conf",
                       "hydra_gateway.cache", "hydra_gateway.key"]
DCN_VMWARE_BLACKLIST = ["inputs.conf", "hydra_metadata.conf",
                        "hydra_session.conf", "ta_vmware_cache.conf", "passwords.conf"]
DCN_HYDRA_BLACKLIST = ["inputs.conf",
                       "hydra_gateway.cache", "hydra_gateway.key"]


def setup_logger(logger_name):
    """
    Creates and returns the logger object with the given name
    :logger_name: Name of the logger
    :return: logger object
    """
    logging.basicConfig(filename=make_splunkhome_path(['var', 'log', 'splunk', 'upgrade_metric_TA_to_new_metric_TA.log']),
                        format='%(asctime)s %(levelname)s %(lineno)d - %(message)s',
                        filemode='a')
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.INFO)
    return logger


logger = setup_logger('upgrade_metric_TA')


def add_failure(message=''):
    """
    Updates the global Failure Flag and add the global Failure Message list
    :message: The mesage to add in the failure messages
    :return: nothing
    """
    global FAILURE_FLAG
    global FAILURE_MESSAGES
    FAILURE_FLAG = True
    if message:
        FAILURE_MESSAGES.append(message)


def log_and_add_failure(error_message, exception):
    """
    Logs the error in the log file and add the failure message to global Failure message list
    :error_message: Message to add in the global failure message list
    :exception: Exception object
    """
    logger.error("{} Error: {}".format(error_message, exception))
    add_failure(error_message)


def directory_exists(*directories):
    """
    Checks the existence of the package in etc/apps
    :param directories: dynamic argument of sequential names of directories
    :return: Boolean based on existance of package
    """
    DIR_PATH = ['etc', 'apps'] + [directory for directory in directories]
    return os.path.isdir(make_splunkhome_path(DIR_PATH))


def metric_ta_exists():
    """
    Checks the existence of the Metric TA packages in etc/apps
    :return: True when the required packages are present.
    """
    required_packages = ["Splunk_TA_vmware_inframon", "SA-Hydra-inframon"]
    missing_packages = list(required_packages)

    for package in required_packages:
        if directory_exists(package):
            missing_packages.remove(package)

    if missing_packages:
        logger.error(
            "{} package(s) are not present in etc/apps directory.".format(missing_packages))
        print("{} pacakge(s) are not present in etc/apps directory, please extract them to etc/apps.".format(missing_packages))
        return False
    else:
        logger.info(
            "The required packages are present in the etc/apps directory.")
        return True


def create_directory(package_name, directory_name='local', permission=0o700):
    """
    Creates directory in the package present in $SPLUNK_HOME/etc/apps with given permission
    :package_name: package in which the directory needs to be made
    :directory_name: name of the directory that needs to be made
    :return: Boolean denoting the status of the operation
    """
    try:
        path = make_splunkhome_path(
            ['etc', 'apps', package_name, directory_name])
        logger.info("Creating {} directory in package: {}".format(
            directory_name, package_name))
        os.mkdir(path, permission)
        logger.info(
            "Successfully created {} directory in package {}".format(directory_name, package_name))
        return True
    except OSError as e:
        error_message = "Error occured while creating {} directory in {} package of etc/apps.".format(
            directory_name, package_name)
        log_and_add_failure(error_message, e)
        return False
    except Exception as e:
        error_message = "Error occured while creating {} directory in {} package of etc/apps.".format(
            directory_name, package_name)
        log_and_add_failure(error_message, e)
        return False


def copy_local_directory_files(source_package, destination_package, blacklist):
    """
    Copies all the files present at the source directory to the destination directory.
    :source_package: Directory of etc/apps from which the files are being copied
    :destination_package: Directory of etc/apps to which the files are copied
    :blacklist: list of files that should not be copied
    """
    source = make_splunkhome_path(['etc', 'apps', source_package, 'local'])
    destination = make_splunkhome_path(
        ['etc', 'apps', destination_package, 'local'])

    def copy_file(file, file_path, destination):
        """
        Copies file from present at given path to the destination path
        :file: name of the file
        :file_path: path of file including the file
        :destination: directory at which the file will be copied
        """
        try:
            logger.info("Copying file {} from {} to {}".format(
                file, file_path, destination))
            shutil.copy(file_path, destination)
            logger.info("Copied file {} from {} to {}".format(
                file, file_path, destination))
            return True
        except OSError as e:
            error_message = "Error occured while copying file from {} to {}.".format(
                file_path, destination)
            log_and_add_failure(error_message, e)
            return False
        except Exception as e:
            error_message = "Error occured while copying file from {} to {}.".format(
                file_path, destination)
            log_and_add_failure(error_message, e)
            return False

    # Recursively walk through the source directory and copy each non-blacklisted file to the destination
    errorneous_files = []
    for root, _, files in os.walk(source):
        for file_name in files:
            if file_name not in blacklist:
                file_path = os.path.join(root, file_name)
                if(not copy_file(file_name, file_path, destination)):
                    errorneous_files.append(file_path)

    if errorneous_files:
        logger.error("Error occured while copying these files from {} to {}: {}".format(
            source, destination, errorneous_files))


def update_file_name(package_name):
    """
    Renames the files present in the local directory of the specififed package
    :package_name: Name of the package whose file needs to be renamed
    """
    renaming_dict = {
        "Splunk_TA_vmware_inframon": {
            "ta_vmware_collection.conf": "inframon_ta_vmware_collection.conf",
            "hydra_node.conf": "inframon_hydra_node.conf",
            "template.conf": "inframon_ta_vmware_template.conf",
            "ta_vmware_config_ssl.conf": "inframon_ta_vmware_config_ssl.conf",
            "pool.conf": "inframon_ta_vmware_pool.conf"
        },
        "SA-Hydra-inframon": {
            "hydra_gateway.conf": "inframon_hydra_gateway.conf"
        }
    }

    filename_mapping = renaming_dict[package_name]
    for old_file_name, new_file_name in filename_mapping.items():
        src = make_splunkhome_path(
            ['etc', 'apps', package_name, 'local', old_file_name])
        dest = make_splunkhome_path(
            ['etc', 'apps', package_name, 'local', new_file_name])
        try:
            if path.isfile(src):
                logger.info("renaming file from {} to {}".format(
                    old_file_name, new_file_name))
                os.rename(src, dest)
                logger.info("renamed file from {} to {}".format(
                    old_file_name, new_file_name))
            else:
                logger.warning("File {} is not present in {}".format(
                    old_file_name, src))
        except OSError as e:
            error_message = "Error occured while renaming the file: {}, present in {}.".format(
                old_file_name, src)
            log_and_add_failure(error_message, e)
        except Exception as e:
            error_message = "Error occured while renaming the file: {}, present in {}.".format(
                old_file_name, src)
            log_and_add_failure(error_message, e)


def create_backup(packages):
    """
    Create backup of given list of packages in etc/apps/backup/VMware-Add-on-<current-time>/<package_name>
    :packages: List of packages for which backup will be created.
    """
    global CURRENT_TIME
    current_time = CURRENT_TIME
    flag = True

    if directory_exists('backup'):
        logger.info("Backup directory is already present")

        # As per short-circuit evaluation, the directory will not be created if it already exists
        if directory_exists('backup', 'VMware-Add-on-' + current_time) or create_directory('backup', 'VMware-Add-on-' + current_time):
            flag = True
    else:
        try:
            logger.info("Creating backup directory")
            os.mkdir(make_splunkhome_path(['etc', 'apps', 'backup']))
            logger.info("backup directory is created")

            if create_directory('backup', 'VMware-Add-on-' + current_time):
                flag = True
        except OSError as e:
            error_message = "Error occured while creating backup directory in etc/apps."
            log_and_add_failure(error_message, e)
            return
        except Exception as e:
            error_message = "Error occured while creating backup directory in etc/apps."
            log_and_add_failure(error_message, e)
            return

    if flag:
        for package_name in packages:
            try:
                logger.info("Taking backup of {}".format(package_name))
                os.rename(make_splunkhome_path(['etc', 'apps', package_name]), make_splunkhome_path(
                    ['etc', 'apps', 'backup', 'VMware-Add-on-' + current_time, package_name]))
                logger.info(
                    "Backup completed succesfully for {} ".format(package_name))
            except OSError as e:
                error_message = "Error occured while taking the backup of {} package.".format(
                    package_name)
                log_and_add_failure(error_message, e)

            except Exception as e:
                error_message = "Error occured while taking the backup of {} package.".format(
                    package_name)
                log_and_add_failure(error_message, e)


def get_user_permission_to_move_ahead(message):
    """
    Called when the script is not able to determine the current Splunk status.
    :message: The message to show to user before prompting for the choice
    :return: True if the user opts to move ahead, else False.
    """
    try:
        print(message)
        print("Enter\n1) To proceed ahead\nAny other key to exit")

        user_choice = input()
        user_choice = int(user_choice)

        if user_choice == 1:
            logger.info("User selected to proceed ahead..")
            return True
        else:
            print(
                "Terminating the script run, please run the upgrade script after stopping the Splunk")
            logger.info(
                "User terminated the script.. Entered input: {}".format(user_choice))
            sys.exit(0)
    except ValueError as e:
        error_message = "Invalid value entered. Entered input: {}.. Terminating the script run, please run the upgrade script after stopping the Splunk".format(
            user_choice)
        print(error_message)
        logger.error("{} Error: {}".format(error_message, e))
        sys.exit(0)
    except Exception as e:
        error_message = "Error occured while getting the user input. Terminating the upgrade script run."
        print(error_message)
        logger.error("{} Error: {}".format(error_message, e))
        sys.exit(0)


def stop_splunk():
    """
    Stops the Splunk
    :return: Boolean variable denoting the operation status
    """
    try:
        splunk_cmd = make_splunkhome_path(['bin', 'splunk'])

        output = subprocess.Popen(
            [splunk_cmd, 'stop'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output_message, output_error = output.communicate()

        if((not output_error) and (b'Done.' in output_message) and (not is_splunk_running())):
            logger.info("Splunk stopped successfully.")
            print("Splunk stopped successfully.")
            return True
    except Exception as e:
        logger.error(
            "Error occured while stopping the Splunk. Error: {}".format(e))

    error_message = "Error occured while stopping the Splunk, please make sure Splunk is stopped before upgrading.."
    return get_user_permission_to_move_ahead(error_message)


def is_splunk_running():
    """
    Checks the Splunk status
    :return: Boolean representing the Splunkd status by running the `splunk status` command by spawning a subprocess.
    """
    try:
        splunk_cmd = make_splunkhome_path(['bin', 'splunk'])

        output = subprocess.Popen([splunk_cmd, 'status'],
                                  stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output_message, output_error = output.communicate()

        if output_error:
            error_message = "Unable to determine status of Splunk. Please make sure Splunk is stopped before upgrading..."
            logger.error("{}. Error: {}".format(output_message, output_error))
            # If user agrees to move ahead, then that means that Splunk is stopped, returning False
            return not get_user_permission_to_move_ahead(error_message)
        else:
            if b"splunkd is running" in output_message:
                logger.info("Splunk service is running.")
                return True
            else:
                logger.info("Splunk service is not running.")
                return False
    except Exception as e:
        error_message = "Error occured while determining status of Splunk. Please make sure Splunk is stopped before proceeding ahead."
        logger.error("{} Error: {}".format(error_message, e))

        # If user agrees to move ahead, then that means that Splunk is stopped, returning False
        return not get_user_permission_to_move_ahead(error_message)


def stop_splunk_wizard():
    """
    Stops the splunk service after obtaining user's choice
    :return: Boolean indicating whether to continue operation or abort it.
    """
    print("Splunk needs to be stopped before running the upgrade script.\nEnter 1 to stop the Splunk\nEnter any other key to terminate upgrade script")
    try:
        user_choice = input()
        user_choice = int(user_choice)

        if user_choice == 1:
            print("Stopping Splunk on this instance..")
            logger.info("Stopping Splunk on this instance..")
            return stop_splunk()
        else:
            raise ValueError()
    except ValueError as e:
        print(
            "Exiting the upgrade script, please stop the Splunk and run the upgrade script again.")
        logger.error(
            "Exiting the upgrade script, please stop the Splunk and run the upgrade script again. Entered input: {}".format(user_choice))
        return False
    except Exception as e:
        print("Error occured while getting input, please stop the Splunk and run the upgrade script again.")
        logger.error(
            "Error occured while getting input, please stop the Splunk and run the upgrade script again. Error: {}".format(e))
        return False


def remove_local_directory(package_name):
    """
    Removes the local directory and its contents of package present in the etc/apps.
    :package_name: package whose `local` directory needs to be removed
    :return: Boolean indicating the status of the operation
    """
    try:
        logger.info("Removing local directory and its contents present in {} package".format(
            str(package_name)))
        path = make_splunkhome_path(['etc', 'apps', package_name, 'local'])
        shutil.rmtree(path)
        logger.info("Removed local directory and its contents present in {} package".format(
            str(package_name)))
        return True
    except OSError as e:
        error_message = "Error occured while removing the contents of the {}/local directory. Error: {}".format(
            package_name, e)
        logger.error(error_message)
        return False
    except Exception as e:
        error_message = "Error occured while removing the contents of the {}/local directory. Error: {}".format(
            package_name, e)
        logger.error(error_message)
        return False


def check_previous_upgrade_status():
    """
    Looks for the metric_ta_upgrade_status.json file present in current directory
    :return: Boolean variables denoting upgrade_needed and backup_stage_achieved flags
    """
    try:
        current_directory = os.getcwd()
        json_file_path = os.path.join(current_directory, UPGRADE_STATUS_FILE)

        if os.path.isfile(json_file_path):

            last_upgrade_status = {}

            with open(json_file_path, 'r') as file_obj:
                last_upgrade_status = json.load(file_obj)

            # Previous upgrade was successful
            if last_upgrade_status['upgrade_successfull'] and not last_upgrade_status['errors']:
                logger.info(
                    "The Scheduler/DCN machine seems to be already upgraded at {}, avoiding rest of the flow.".format(last_upgrade_status['time']))
                print("The Scheduler/DCN machine seems to be already upgraded at {}, the upgrade sequence will not run.".format(
                    last_upgrade_status['time']))
                return False, last_upgrade_status['backup_state_reached']
            else:
                logger.info("The last upgrade performed at {} had some failures, flag: {} and errors: {}".format(
                    last_upgrade_status['time'], last_upgrade_status['upgrade_successfull'], last_upgrade_status['errors']))
                print("The last upgrade performed at {} had some failures..".format(
                    last_upgrade_status['time']))

                if not last_upgrade_status['backup_state_reached']:
                    # Backup stage was not achieved in the last upgrade sequence
                    # Clean the local directory and run the upgrade sequence again
                    logger.info(
                        "Reverting the changes and running the upgrade sequence again...")
                    print(
                        "Reverting the changes and running the upgrade sequence again...")
                    # Previous upgrade had some failures, revert the changes..
                    changes_reverted = True
                    packages = ['Splunk_TA_vmware_inframon',
                                'SA-Hydra-inframon']
                    for package in packages:
                        if directory_exists(package, 'local'):
                            if not remove_local_directory(package):
                                changes_reverted = False
                                print(
                                    "Error occured while removing the local directory from {} package in etc/apps. Please remove the local directory manually and run the upgrade script again".format(package))
                                logger.error(
                                    "Error occured while removing the local directory from {} package".format(package))
                        else:
                            error_message = "local directory was not found in {} package of etc/apps.".format(
                                package)
                            logger.info(error_message)

                    return changes_reverted, last_upgrade_status['backup_state_reached']
                else:
                    # Backup stage is achieved in the last upgrade, only run the upgrade stage
                    return True, last_upgrade_status['backup_state_reached']

        else:
            # Either user has changed the place of running the upgrade script or this is really a fresh upgrade
            logger.info(
                "The JSON file for the status was not found, this is the fresh upgrade.")
            return True, False
    except KeyError as e:
        error_message = "Error occured while reading last upgrade status from {} file. Considering as fresh upgrade...".format(
            UPGRADE_STATUS_FILE)
        print(error_message)
        logger.error("{} Error: {}".format(error_message, e))
        return True, False
    except ValueError as e:
        error_message = "Error occured while reading last upgrade status from {} file. Considering as fresh upgrade...".format(
            UPGRADE_STATUS_FILE)
        print(error_message)
        logger.error("{} Error: {}".format(error_message, e))
        return True, False
    except Exception as e:
        error_message = "Error occured while reading last upgrade status from {} file. Considering as fresh upgrade...".format(
            UPGRADE_STATUS_FILE)
        print(error_message)
        logger.error("{} Error: {}".format(error_message, e))
        return True, False


def run_prechecks():
    """
    Run the prechecks for the upgrade script
    :return: boolean denoting whether to do upgrade or not.
    """
    global BACKUP_STATE_ACHIEVED
    if is_splunk_running():
        if not stop_splunk_wizard():
            return False

    upgrade_needed, BACKUP_STATE_ACHIEVED = check_previous_upgrade_status()

    if upgrade_needed and not BACKUP_STATE_ACHIEVED:
        # This is either a fresh upgrade or an upgrade who didn't reach in BACKUP state
        if not directory_exists("Splunk_TA_vmware") or not directory_exists("SA-Hydra"):
            logger.info(
                "The Scheduler/DCN machine does not have at least one of the packages from Splunk_TA_vmware and SA-Hydra, avoiding rest of the flow.")
            print("The Scheduler/DCN machine does not have one of the at least one package from Splunk_TA_vmware and SA-Hydra, the upgrade sequence will not run.")
            return False
    elif upgrade_needed and BACKUP_STATE_ACHIEVED:
        # This means only backup state is facing the failure
        logger.info("Only the backup phase is to be performed.")
        print("Seems that the Scheduler/DCN component is already upgraded. Performing backup of packages.")
    else:
        # This will occur when last upgrade was successful and user tries to run the script again.
        return False

    return metric_ta_exists()


def component_selection_and_upgrade():
    """
    Asks user to select the component on which the script is running.
    :return: 'SCH' or 'DCN' based on the user selection
    """
    print("Please select the component on which the script is running.\nPress\n1) Scheduler\n2) DCN\nEnter your choice: ")

    try:
        choice = input()

        if int(choice) == 1:
            message = "Running the Upgrade sequence for the scheduler.."
            logger.info(message)
            print(message)
            upgrade_scheduler()
        elif int(choice) == 2:
            message = "Running the Upgrade sequence for the DCN.."
            logger.info(message)
            print(message)
            upgrade_dcn()
        else:
            raise ValueError()

    except ValueError as e:
        print("Please enter Valid Input. Entered input: {}".format(choice))
        logger.info(
            "Invalid Input entered while selecting the component. Entered input: {}".format(choice))

        component_selection_and_upgrade()
    except Exception as e:
        print("Error occured while rendering input. Please enter appropriate input.")
        logger.info(
            "Exception occured while getting the input. Exception: {}".format(str(e)))

        component_selection_and_upgrade()


def write_json_file(upgrade_status_dict):
    """
    Writes the JSON file with the upgrade status
    :upgrade_status_dict: The dictionary having values denoting upgrade status
    """
    try:
        current_directory = os.getcwd()
        json_file_path = os.path.join(current_directory, UPGRADE_STATUS_FILE)

        with open(json_file_path, 'w') as fileObj:
            json.dump(upgrade_status_dict, fileObj)
    except Exception as e:
        print("Error occured while storing the status in the {} file. Error: {}".format(
            UPGRADE_STATUS_FILE, e))
        logger.error("Error occured while storing the status in the {} file.".format(
            UPGRADE_STATUS_FILE))


def upgrade_scheduler():
    """
    Upgrades the instance by following upgrade steps of Scheduler
    """
    global FAILURE_FLAG
    global FAILURE_MESSAGES
    global BACKUP_STATE_ACHIEVED

    if not BACKUP_STATE_ACHIEVED:
        if create_directory('Splunk_TA_vmware_inframon') and create_directory('SA-Hydra-inframon'):
            copy_local_directory_files(
                'Splunk_TA_vmware', 'Splunk_TA_vmware_inframon', SCH_VMWARE_BLACKLIST)
            update_file_name('Splunk_TA_vmware_inframon')
            copy_local_directory_files(
                'SA-Hydra', 'SA-Hydra-inframon', SCH_HYDRA_BLACKLIST)
            update_file_name('SA-Hydra-inframon')

    if FAILURE_FLAG or FAILURE_MESSAGES:
        upgrade_status_dict = {
            "time": CURRENT_TIME,
            "errors": FAILURE_MESSAGES,
            "upgrade_successfull": False,
            "backup_state_reached": False
        }
        write_json_file(upgrade_status_dict)
        print("Errors occurred while upgrading the scheduler. Please run the script again. Errors: {}".format(
            FAILURE_MESSAGES))
        logger.error(
            "Errors occured while upgrading the scheduler. Errors: {}".format(FAILURE_MESSAGES))
    else:
        create_backup(["Splunk_TA_vmware", "SA-Hydra"])

        if FAILURE_FLAG or FAILURE_MESSAGES:
            upgrade_status_dict = {
                "time": CURRENT_TIME,
                "errors": FAILURE_MESSAGES,
                "upgrade_successfull": False,
                "backup_state_reached": True
            }
            write_json_file(upgrade_status_dict)
            print("Errors occurred while taking the backup of the Splunk_TA_vmware and SA-Hydra packages. Please run the script again. Errors: {}".format(
                FAILURE_MESSAGES))
            logger.error(
                "Errors occurred while taking the backup of the Splunk_TA_vmware and SA-Hydra packages. Please run the script again. Errors: {}".format(FAILURE_MESSAGES))
        else:
            upgrade_status_dict = {
                "time": CURRENT_TIME,
                "errors": FAILURE_MESSAGES,
                "upgrade_successfull": True,
                "backup_state_reached": True
            }
            write_json_file(upgrade_status_dict)
            print("Scheduler is successfully upgraded to latest Metric TA.")
            logger.info(
                "Scheduler is successfully upgraded to latest Metric TA.")


def upgrade_dcn():
    """
    Upgrades the instance by following upgrade steps of DCN
    """
    global FAILURE_FLAG
    global FAILURE_MESSAGES
    global BACKUP_STATE_ACHIEVED

    if not BACKUP_STATE_ACHIEVED:
        if create_directory('Splunk_TA_vmware_inframon') and create_directory('SA-Hydra-inframon'):
            copy_local_directory_files(
                'Splunk_TA_vmware', 'Splunk_TA_vmware_inframon', DCN_VMWARE_BLACKLIST)
            copy_local_directory_files(
                'SA-Hydra', 'SA-Hydra-inframon', DCN_HYDRA_BLACKLIST)
            update_file_name('SA-Hydra-inframon')

    if FAILURE_MESSAGES and FAILURE_MESSAGES:
        # Do not create backup and dump the status to JSON file
        upgrade_status_dict = {
            "time": CURRENT_TIME,
            "errors": FAILURE_MESSAGES,
            "upgrade_successfull": False,
            "backup_state_reached": False
        }
        write_json_file(upgrade_status_dict)
        print("Errors occurred while upgrading the DCN. Please run the script again. Errors: {}".format(
            FAILURE_MESSAGES))
        logger.error(
            "Errors occured while upgrading the DCN. Errors: {}".format(FAILURE_MESSAGES))
    else:
        create_backup(["Splunk_TA_vmware", "SA-Hydra"])

        if FAILURE_FLAG or FAILURE_MESSAGES:
            # Errors occured while creating backup, dump the status to JSON file
            upgrade_status_dict = {
                "time": CURRENT_TIME,
                "errors": FAILURE_MESSAGES,
                "upgrade_successfull": False,
                "backup_state_reached": True
            }
            write_json_file(upgrade_status_dict)
            print("Errors occurred while taking the backup of the Splunk_TA_vmware and SA-Hydra packages. Please run the script again. Errors: {}".format(
                FAILURE_MESSAGES))
            logger.error(
                "Errors occurred while taking the backup of the Splunk_TA_vmware and SA-Hydra packages. Please run the script again. Errors: {}".format(FAILURE_MESSAGES))

        else:
            upgrade_status_dict = {
                "time": CURRENT_TIME,
                "errors": FAILURE_MESSAGES,
                "upgrade_successfull": True,
                "backup_state_reached": True
            }
            write_json_file(upgrade_status_dict)
            print("DCN is successfully upgraded to latest Metric TA.")
            logger.info(
                "DCN is successfully upgraded to latest Metric TA.")


def main():

    logger.info("Initiating the upgrade script for the metric TA")
    if run_prechecks():
        component_selection_and_upgrade()


if __name__ == "__main__":
    main()